#!/bin/bash
accumsize=$(($1))

if [ "$accumsize" -lt "10" ]; then
	echo Aborted!  Specify size range 10-64.
	echo
	exit
fi

double=$(( $accumsize*2 ))

scoredir=test-s${accumsize}

# Sort the candidates
sort -g -k1 -k2 -k3 "${scoredir}/maxiter-xs-s${accumsize}.out" >"${scoredir}/sorted-xs-s${accumsize}.out"
sort -g -k1 -k2 -k3 "${scoredir}/maxiter-xo-s${accumsize}.out" >"${scoredir}/sorted-xo-s${accumsize}.out"

if [[ $? != 0 ]]; then
	echo Aborted!  Test-set not found.
	echo
	exit
fi


function extract_score {
	if [ -f $1 ]; then
		grep "length=" $1 |tail -n1 |awk '{ printf "%3s%1s", $2, toupper(substr($3,1,1)) }'
	else
		printf "    "
	fi
}


printf "\n\n    Xorshift${double}+ PractRand Score Table\n\nCombination    Word1   Word2  Byte%02d  Byte%02d   Byte2   Byte1    Bit\n====================================================================\n" $((${accumsize}-8)) $(((${accumsize}-8)/2)) |tee "PRscore-s${accumsize}.txt"

# Xorshift - Read one candidate line at a time from the file, picking out each 'a' 'b' 'c' combination
while IFS=", \t" read a b c line
do
# Convert to integers
	a=$(($a)); b=$(($b)); c=$(($c))

# Extract the candidate score for each test variant
	word1=$( extract_score "${scoredir}/PRscore-xs-s${accumsize}word1-a${a}b${b}c${c}.out" )
	word2=$( extract_score "${scoredir}/PRscore-xs-s${accumsize}word2-a${a}b${b}c${c}.out" )
	bytems=$( extract_score "${scoredir}/PRscore-xs-s${accumsize}byte$((${accumsize}-8))-a${a}b${b}c${c}.out" )
	if [ $accumsize -gt 13 ]; then  # Duplicate filenames
		bytehs=$( extract_score "${scoredir}/PRscore-xs-s${accumsize}byte$(((${accumsize}-8)/2))-a${a}b${b}c${c}.out" )
	else
		bytehs="    "
	fi
	if [ $accumsize -gt 10 ]; then  # Duplicate filenames
		byte2=$( extract_score "${scoredir}/PRscore-xs-s${accumsize}byte2-a${a}b${b}c${c}.out" )
	else
		byte2="    "
	fi
	byte1=$( extract_score "${scoredir}/PRscore-xs-s${accumsize}byte1-a${a}b${b}c${c}.out" )
	bit=$( extract_score "${scoredir}/PRscore-xs-s${accumsize}bit-a${a}b${b}c${c}.out" )

# Output the candidate scores
	printf " [%2d %2d %2d]    $word1    $word2    $bytems    $bytehs    $byte2    $byte1    $bit\n" $a $b $c |tee -a "PRscore-s${accumsize}.txt"

done < "${scoredir}/sorted-xs-s${accumsize}.out"


printf "\n\n    Xoroshiro${double}+ PractRand Score Table\n\nCombination    Word1   Word2  Byte%02d  Byte%02d   Byte2   Byte1    Bit\n====================================================================\n" $((${accumsize}-8)) $(((${accumsize}-8)/2)) |tee -a "PRscore-s${accumsize}.txt"

# Xoroshiro - Read one candidate line at a time from the file, picking out each 'a' 'b' 'c' combination
while IFS=", \t" read a b c line
do
# Convert to integers
	a=$(($a)); b=$(($b)); c=$(($c))

# Extract the candidate score for each test variant
	word1=$( extract_score "${scoredir}/PRscore-xo-s${accumsize}word1-a${a}b${b}c${c}.out" )
	word2=$( extract_score "${scoredir}/PRscore-xo-s${accumsize}word2-a${a}b${b}c${c}.out" )
	bytems=$( extract_score "${scoredir}/PRscore-xo-s${accumsize}byte$((${accumsize}-8))-a${a}b${b}c${c}.out" )
	if [ $accumsize -gt 13 ]; then  # Duplicate filenames
		bytehs=$( extract_score "${scoredir}/PRscore-xo-s${accumsize}byte$(((${accumsize}-8)/2))-a${a}b${b}c${c}.out" )
	else
		bytehs="    "
	fi
	if [ $accumsize -gt 10 ]; then  # Duplicate filenames
		byte2=$( extract_score "${scoredir}/PRscore-xo-s${accumsize}byte2-a${a}b${b}c${c}.out" )
	else
		byte2="    "
	fi
	byte1=$( extract_score "${scoredir}/PRscore-xo-s${accumsize}byte1-a${a}b${b}c${c}.out" )
	bit=$( extract_score "${scoredir}/PRscore-xo-s${accumsize}bit-a${a}b${b}c${c}.out" )

# Output the candidate scores
	printf " [%2d %2d %2d]    $word1    $word2    $bytems    $bytehs    $byte2    $byte1    $bit\n" $a $b $c |tee -a "PRscore-s${accumsize}.txt"

done < "${scoredir}/sorted-xo-s${accumsize}.out"
