#!/bin/bash
accumsize=$(($1))

if [ $accumsize -lt 10 ]; then
	echo Aborted!  Specify size range of 10-32.
	echo
	exit
fi

double=$(( $accumsize*2 ))
top=$(( $accumsize-1 ))
lowhalf=$(( $accumsize/2-1 ))
#highhalf=$(( $accumsize-$lowhalf ))
highhalf=$(( $accumsize/2+1 ))

scoredir=test-s${accumsize}

mkdir "${scoredir}"

if [[ $? != 0 ]]; then
	echo Aborted!  Nothing done.
	echo
	exit
fi

printf "Compiling iterators for word size of ${accumsize} ...\n" |tee "${scoredir}/maxiter-xs-s${accumsize}.log" |tee "${scoredir}/maxiter-xo-s${accumsize}.log"
if [ $accumsize -lt 12 ]; then
	gcc -o "${scoredir}/maxiter-xs" "src/xorshift-fullperiod64.c" -O3 -DACCUM_SIZE=${accumsize} |tee -a "${scoredir}/maxiter-xs-s${accumsize}.log"
	gcc -o "${scoredir}/maxiter-xo" "src/xoroshiro-fullperiod64.c" -O3 -DACCUM_SIZE=${accumsize} |tee -a "${scoredir}/maxiter-xo-s${accumsize}.log"
else
	gcc -o "${scoredir}/maxiter-xs" "src/xorshift-fullperiod64.c" -O3 -DACCUM_SIZE=${accumsize} -DSTATUS_ON=1 | tee -a "${scoredir}/maxiter-xs-s${accumsize}.log"
	gcc -o "${scoredir}/maxiter-xo" "src/xoroshiro-fullperiod64.c" -O3 -DACCUM_SIZE=${accumsize} -DSTATUS_ON=1 |tee -a "${scoredir}/maxiter-xo-s${accumsize}.log"
fi

if [[ $? != 0 ]]; then
	echo Aborted!  Compile error.
	echo
	exit
fi

# Check for spare mips - initial sample point
read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
activeafter=$(( $user + $nice + $system + $softirq + $steal ))
totalafter=$(( $activeafter + $idle + $iowait ))


printf "Launching tasks to brute force find full-period combinations of Xorshift${double}+ and Xoroshiro${double}+ random generators at " |tee -a "${scoredir}/maxiter-xs-s${accumsize}.log" |tee -a "${scoredir}/maxiter-xo-s${accumsize}.log"
date +"%F %T" |tee -a "${scoredir}/maxiter-xs-s${accumsize}.log" |tee -a "${scoredir}/maxiter-xo-s${accumsize}.log"

for a in `seq 1 $top`; do
#for b in `seq 1 $top`; do
	printf "\nStart combinations [${a} xx xx]" |tee -a "${scoredir}/maxiter-xs-s${accumsize}.log" |tee -a "${scoredir}/maxiter-xo-s${accumsize}.log"
	nice -11 stdbuf -o L "${scoredir}/maxiter-xs" ${a} >>"${scoredir}/maxiter-xs-s${accumsize}.out" 2>>"${scoredir}/maxiter-xs-s${accumsize}.log" &
	nice -11 stdbuf -o L "${scoredir}/maxiter-xo" ${a} >>"${scoredir}/maxiter-xo-s${accumsize}.out" 2>>"${scoredir}/maxiter-xo-s${accumsize}.log" &

# Check for spare mips
	if [ $accumsize -gt 14 ]; then
		while true
		do
			activebefore=$activeafter
			totalbefore=$totalafter
			sleep 3
			read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
			activeafter=$(( $user + $nice + $system + $softirq + $steal ))
			totalafter=$(( $activeafter + $idle + $iowait ))

			cpuload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
#echo cpuload=${cpuload}
			if [ $cpuload -lt 50 ]; then  # Suited to dual threaded processor cores
				break
			fi
		done
	fi
#done
done

#printf "Launching $(($highhalf * 2)) processes to brute force find full-period combinations of Xoroshiro${double}+ random generator"
#for a in `seq $lowhalf $top`; do
#for c in `seq 1 $highhalf`; do
#	printf "\nStart combinations for [${a} -- ${c}]"
#	nice -11 stdbuf -o L "${scoredir}/maxiter" ${a} ${c} >"${scoredir}/maxiter-s${accumsize}-a${a}c${c}.out" &
#	sleep 1
#done
#done
#date +"%F %T"
echo
