#!/bin/bash
accumsize=$(($1))

if [ $accumsize -lt 10 ]; then
	echo Aborted!  Specify size range 10-64.
	echo
	exit
fi

double=$(( $accumsize*2 ))

scoredir=test-s${accumsize}

if [ ! -f "${scoredir}/maxiter-xs-s${accumsize}.out" ] && [ ! -f "${scoredir}/maxiter-xo-s${accumsize}.out" ]; then
	echo Aborted!  Test-set not found.
	echo
	exit
fi

# Check for spare mips - initial sample point
read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
activeafter=$(( $user + $nice + $system + $softirq + $steal ))
totalafter=$(( $activeafter + $idle + $iowait ))


printf "\nBEGIN\nPractRand scoring selected combinations of Xorshift${double}+ and Xoroshiro${double}+ random generators.\n" |tee -a "${scoredir}/PRscore-xs-s${accumsize}.log" |tee -a "${scoredir}/PRscore-xo-s${accumsize}.log"


# Xorshift - Read one candidate line at a time from the file, picking out each 'a' 'b' 'c' combination
while IFS=", \t" read a b c line
do
# Convert to integers
	a=$(($a)); b=$(($b)); c=$(($c))

	printf "Scoring Xorshift${double}+ combination [${a} ${b} ${c}] at " |tee -a "${scoredir}/PRscore-xs-s${accumsize}.log"
	date +"%F %T" |tee -a "${scoredir}/PRscore-xs-s${accumsize}.log"

# Variants:  Full sized generator word (minus the lsb for quality), truncated byte (adjustable position), truncated bit (very slow).
	nice -11 src/testcase-xorshift.sh ${accumsize} ${a} ${b} ${c} 1 "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	nice -11 src/testcase-xorshift.sh ${accumsize} ${a} ${b} ${c} 2 "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	if [ $accumsize -lt 25 ]; then
		nice -11 src/bytecase-xorshift.sh ${accumsize} ${a} ${b} ${c} $((${accumsize}-8)) "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	fi
	if [ $accumsize -gt 13 ] && [ $accumsize -lt 25 ]; then
		nice -11 src/bytecase-xorshift.sh ${accumsize} ${a} ${b} ${c} $(((${accumsize}-8)/2)) "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	fi
	if [ $accumsize -gt 10 ] && [ $accumsize -lt 25 ]; then
		nice -11 src/bytecase-xorshift.sh ${accumsize} ${a} ${b} ${c} 2 "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
		nice -11 src/bytecase-xorshift.sh ${accumsize} ${a} ${b} ${c} 1 "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	fi
	if [ $accumsize -lt 19 ]; then
		nice -11 src/bitcase-xorshift.sh ${accumsize} ${a} ${b} ${c} "${scoredir}" >>"${scoredir}/PRscore-xs-s${accumsize}.log" &
	fi

# Check for spare mips
	while true
	do
		activebefore=$activeafter
		totalbefore=$totalafter
		sleep 3
		read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
		activeafter=$(( $user + $nice + $system + $softirq + $steal ))
		totalafter=$(( $activeafter + $idle + $iowait ))

		cpuload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
#echo cpuload=${cpuload}
		if [ $cpuload -lt 50 ]; then  # Suited to dual threaded processor cores
			break
		fi
	done

done < "${scoredir}/maxiter-xs-s${accumsize}.out"


# Xoroshiro - Read one candidate line at a time from the file, picking out each 'a' 'b' 'c' combination
while IFS=", \t" read a b c line
do
# Convert to integers
	a=$(($a)); b=$(($b)); c=$(($c))

	printf "Scoring Xoroshiro${double}+ combination [${a} ${b} ${c}] at " |tee -a "${scoredir}/PRscore-xo-s${accumsize}.log"
	date +"%F %T" |tee -a "${scoredir}/PRscore-xo-s${accumsize}.log"

# Variants:  Full sized generator word (minus the lsb for quality), truncated byte (adjustable position), truncated bit (very slow).
	nice -11 src/testcase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} 1 "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	nice -11 src/testcase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} 2 "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	if [ $accumsize -lt 25 ]; then
		nice -11 src/bytecase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} $((${accumsize}-8)) "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	fi
	if [ $accumsize -gt 13 ] && [ $accumsize -lt 25 ]; then
		nice -11 src/bytecase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} $(((${accumsize}-8)/2)) "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	fi
	if [ $accumsize -gt 10 ] && [ $accumsize -lt 25 ]; then
		nice -11 src/bytecase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} 2 "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
		nice -11 src/bytecase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} 1 "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	fi
	if [ $accumsize -lt 19 ]; then
		nice -11 src/bitcase-xoroshiro.sh ${accumsize} ${a} ${b} ${c} "${scoredir}" >>"${scoredir}/PRscore-xo-s${accumsize}.log" &
	fi

# Check for spare mips
	while true
	do
		activebefore=$activeafter
		totalbefore=$totalafter
		sleep 3
		read cpu user nice system idle iowait irq softirq steal guest <"/proc/stat"
		activeafter=$(( $user + $nice + $system + $softirq + $steal ))
		totalafter=$(( $activeafter + $idle + $iowait ))

		cpuload=$(( 101 * ($activeafter - $activebefore) / ($totalafter - $totalbefore) ))
#echo cpuload=${cpuload}
		if [ $cpuload -lt 50 ]; then  # Suited to dual threaded processor cores
			break
		fi
	done

done < "${scoredir}/maxiter-xo-s${accumsize}.out"
