#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <time.h>


//#define  ACCUM_SIZE  16   // Passed in as -D gcc parameter. (Min=9, Max=32) Bit width of generated random numbers.
#define  ACCUM_MASK  (((1UL<<(ACCUM_SIZE-1))-1)<<1|1)   // Bit of a shuffle to work on all 64 bits :)
#define  CNT_LIMIT  (((1UL<<(ACCUM_SIZE*2-1))-1)<<1|1)


//  Find max iterations of every combination of constants in the Xoroshiro random number generator.
int  main(int argc, char* argv[])
{
	static uint32_t  s0, s1;  // Must be static to prevent optimiser vanishing the whole iterator!
	uint64_t  count;
	uint32_t  work;
	int  constant_a, constant_b, constant_c, candidates = 0;
	time_t  enddate;
	struct timespec  starttime, endtime;
	char  datebuff[40];


	if( argc != 2 )  {
//		fputs( "\nError!  Three constants required.\n", stderr );
		fprintf( stderr, "Error!  Parameters required.\n" );
		exit(1);
	}
	constant_a = atoi( argv[1] );
//	constant_b = atoi( argv[2] );
//	constant_c = atoi( argv[3] );
//	printf( "\nConfirm constants: argv[1]=%d argv[2]=%d argv[3]=%d\n", constant_a, constant_b, constant_c );
	if( constant_a < 1 || constant_a > (ACCUM_SIZE-1) )  {
		fprintf( stderr, "Error!  Value (1-%d) for constantA required.\n", (ACCUM_SIZE-1) );
		exit(2);
	}

	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &starttime);  // POSIX

//
// Try each combination
//
//for( constant_a = 1; constant_a < ACCUM_SIZE; constant_a++ )  {
for( constant_b = 1; constant_b < ACCUM_SIZE; constant_b++ )  {
for( constant_c = 1; constant_c < ACCUM_SIZE; constant_c++ )  {
	s0 = 1; s1 = 0;   // Seed the state array.  Seed MUST not be all zero.
	count = 0;

// Brute force iterate to find full length repeats (2^32 - 1 for 16-bit accumulators, 32-bit state)
	do {
		count++;

		work = (s0 ^ (s0 << constant_a)) & ACCUM_MASK;
		s0 = s1;
		s1 = (work ^ s1 ^ (work >> constant_b) ^ (s1 >> constant_c)) & ACCUM_MASK;

		if( s0 == 1 && s1 == 0 )
			break;
	} while( count < CNT_LIMIT );

#ifdef STATUS_ON
    clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &endtime);  // POSIX
	enddate = time(NULL);
	strftime( datebuff, sizeof datebuff, "%T", localtime(&enddate) );
	fprintf( stderr, "\n%s Xorshift  [%2d %2d %2d] iterated %lx, using %ld seconds", datebuff, constant_a, constant_b, constant_c, count, endtime.tv_sec - starttime.tv_sec );

	starttime.tv_sec = endtime.tv_sec;
#endif

	if( count == CNT_LIMIT )  {
		printf( " %2d %2d %2d   ", constant_a, constant_b, constant_c );
		printf( "%02x%02x%02x   %lx\n", constant_a, constant_b, constant_c, CNT_LIMIT );
		candidates++;
	}
}}
//fprintf( stderr, "\n%s Xorshift [%2d xx xx] completed.\n", datebuff, constant_a );
	return 0;
}
